package com.bwie.auth.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.auth.feign.AuthFeign;
import com.bwie.auth.service.AuthService;
import com.bwie.common.constants.JwtConstants;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.User;
import com.bwie.common.domain.response.JwtResponse;
import com.bwie.common.exception.CustomException;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.amqp.rabbit.core.RabbitTemplate;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.HashMap;
import java.util.UUID;
import java.util.concurrent.TimeUnit;

@Service
@Slf4j
public class AuthServiceImpl implements AuthService {
    @Autowired
    private RabbitTemplate rabbitTemplate;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HttpServletRequest request;
    @Autowired
    private AuthFeign authFeign;
    /**
     * 登录
     * @param user
     * @return
     */
    @Override
    public JwtResponse login(User user) {
        User data = authFeign.login(user).getData();
        if(null == data){
            throw new CustomException("用户名不存在");
        }
        if(!data.getUpwd().equals(user.getUpwd())){
            throw new CustomException("密码错误");
        }
        String key = UUID.randomUUID().toString().replaceAll("-", "");
        redisTemplate.opsForValue().set(TokenConstants.LOGIN_TOKEN_KEY+key, JSON.toJSONString(data),30, TimeUnit.MINUTES);
        HashMap<String, Object> map = new HashMap<>();
        map.put(JwtConstants.USER_KEY,key);
        String token = JwtUtils.createToken(map);
        JwtResponse jwtResponse = new JwtResponse();
        jwtResponse.setToken(token);
        jwtResponse.setExistTime("30MIN");
        return jwtResponse;
    }
    /**
     * 身份
     * @return
     */
    @Override
    public User getinfo() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String jsonStr = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(jsonStr, User.class);
        return user;
    }
    /**
     * 退出
     * @return
     */
    @Override
    public void logout() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        redisTemplate.delete(TokenConstants.LOGIN_TOKEN_KEY +userKey);
    }

    /**
     * 注册
     * @param user
     */
    @Override
    public void insertUser(User user) {
        User data = authFeign.login(user).getData();
        if(null != data){
            throw new CustomException("用户名已存在");
        }
        // 添加注册成功
        Integer i = authFeign.insertUser(user).getData();
        if(i>0){
            log.info("发送消息,恭喜你注册成功");
            //发短信
            redisTemplate.opsForValue().set(user.getPhone()+"111","发送消息,恭喜你注册成功");
            rabbitTemplate.convertAndSend("snedPhone","发送消息,恭喜你注册成功",message -> {
                message.getMessageProperties().setMessageId(UUID.randomUUID().toString());
                return message;
            });
        }
    }
}
