package com.bwie.gateway.filters;

import com.bwie.common.constants.TokenConstants;
import com.bwie.common.utils.JwtUtils;
import com.bwie.common.utils.StringUtils;
import com.bwie.gateway.config.IgnoreWhiteConfig;
import com.bwie.gateway.utils.GatewayUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.cloud.gateway.filter.GatewayFilter;
import org.springframework.cloud.gateway.filter.GatewayFilterChain;
import org.springframework.core.Ordered;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.http.HttpStatus;
import org.springframework.http.server.reactive.ServerHttpRequest;
import org.springframework.stereotype.Component;
import org.springframework.web.server.ServerWebExchange;
import reactor.core.publisher.Mono;

import java.util.List;
import java.util.concurrent.TimeUnit;

@Component
public class MyFilters implements GatewayFilter, Ordered {
    @Autowired
    private IgnoreWhiteConfig ignoreWhiteConfig;
    @Autowired
    private StringRedisTemplate  redisTemplate;


    @Override
    public Mono<Void> filter(ServerWebExchange exchange, GatewayFilterChain chain) {
        // 获取到白名单
        List<String> whites = ignoreWhiteConfig.getWhites();
        // 获取当前的 请求头
        ServerHttpRequest request = exchange.getRequest();
        // 获取到路径
        String path = request.getURI().getPath();
        // 判断 路径在白名单中存不存在
        if(StringUtils.matches(path,whites)){
            //放行
            return chain.filter(exchange);
        }
        // 获取 token
        String token = request.getHeaders().getFirst(TokenConstants.TOKEN);
        // 非空验证
        if(StringUtils.isBlank(token)){
            // 错误提示
            return GatewayUtils.errorResponse(exchange,"token不存在", HttpStatus.UNAUTHORIZED);
        }
        // 合法性
        String userKey = JwtUtils.getUserKey(token);
        if(!redisTemplate.hasKey(TokenConstants.LOGIN_TOKEN_KEY+userKey)){
            return GatewayUtils.errorResponse(exchange,"token过期");
        }
        // 续期
        redisTemplate.expire(TokenConstants.LOGIN_TOKEN_KEY+userKey,30, TimeUnit.MINUTES);
        //放行
        return chain.filter(exchange);
    }

    @Override
    public int getOrder() {
        return 0;
    }
}
