package com.bwie.es.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.domain.Goods;
import com.bwie.es.feign.EsFeign;
import com.bwie.es.service.EsService;
import org.elasticsearch.action.bulk.BulkRequest;
import org.elasticsearch.action.delete.DeleteRequest;
import org.elasticsearch.action.index.IndexRequest;
import org.elasticsearch.client.RequestOptions;
import org.elasticsearch.client.RestHighLevelClient;
import org.elasticsearch.common.xcontent.XContentType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.util.List;

@Service
public class EsServiceImpl implements EsService {
    @Autowired
    private EsFeign esFeign;
    @Autowired
    private RestHighLevelClient restHighLevelClient;
    public static final String INDEX_NAME = "apply";

    /**
     * 全量同步,批量添加
     * @param data
     */
    @Override
    public void plAdd(List<Goods> data) {
        try {
            BulkRequest bulkRequest = new BulkRequest(INDEX_NAME);
            data.forEach(goods -> {
                bulkRequest.add(
                        new IndexRequest()
                                .id(goods.getGid()+"")
                                .source(JSON.toJSONString(goods), XContentType.JSON)
                );
            });
            restHighLevelClient.bulk(bulkRequest, RequestOptions.DEFAULT);
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

    /**
     * 防止重复添加
     */
    @Override
    public void delAllData(List<Goods> data) {

        try {
            BulkRequest bulkRequest = new BulkRequest();
            data.forEach(goods -> {
                bulkRequest.add(
                        new DeleteRequest(INDEX_NAME)
                                .id(goods.getGid()+"")
                );
            });
            restHighLevelClient.bulk(bulkRequest,RequestOptions.DEFAULT);
        } catch (Exception e) {
            throw new RuntimeException(e);
        }

    }
}
