package com.bwie.expressage.controller;

import com.alibaba.fastjson.JSON;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Goods;
import com.bwie.common.domain.Rob;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.*;
import com.bwie.common.domain.response.RobListVO;
import com.bwie.common.domain.response.ShopListVO;
import com.bwie.common.result.PageResult;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.expressage.service.ExService;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@RestController
public class ExController {
    @Autowired
    private ExService exService;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HttpServletRequest request;
    /**
     * 秒杀列表
     * @param robListDTO
     * @return
     */
    @PostMapping("robList")
    public Result<PageResult<RobListVO>> robList(@RequestBody RobListDTO robListDTO){
        PageHelper.startPage(robListDTO.getPageNum(),robListDTO.getPageSzie());
        User user = getinfo();
        if(null == user){
            return Result.error("请先登录");
        }
        if(user.getRole()==1){
            // 普通用户
            robListDTO.setUid(user.getUid());
        }
        PageInfo<RobListVO> info = new PageInfo<>(exService.robList(robListDTO));
        return PageResult.toResult(info.getTotal(),info.getList());
    }
    /**
     * 商品列表
     * @param goodsListDTO
     * @return
     */
    @PostMapping("goodsList")
    public Result<PageResult<Goods>> goodsList(@RequestBody GoodsListDTO goodsListDTO){
        PageHelper.startPage(goodsListDTO.getPageNum(),goodsListDTO.getPageSzie());
        User user = getinfo();
        if(null == user){
            return Result.error("请先登录");
        }
        if(user.getRole()==1){
            // 普通用户
            goodsListDTO.setUid(user.getUid());
        }
        PageInfo<Goods> info = new PageInfo<>(exService.goodsList(goodsListDTO));
        return PageResult.toResult(info.getTotal(),info.getList());
    }
    /**
     * 购物车列表
     * @param shopListDTO
     * @return
     */
    @PostMapping("shopList")
    public Result<PageResult<ShopListVO>> shopList(@RequestBody ShopListDTO shopListDTO){
        PageHelper.startPage(shopListDTO.getPageNum(),shopListDTO.getPageSzie());
        User user = getinfo();
        if(null == user){
            return Result.error("请先登录");
        }
        if(user.getRole()==1){
            // 普通用户
            shopListDTO.setUid(user.getUid());
        }
        PageInfo<ShopListVO> info = new PageInfo<>(exService.shopList(shopListDTO));
        return PageResult.toResult(info.getTotal(),info.getList());
    }

    /**
     * 添加购物车表
     * @param insertShopDTO
     * @return
     */
    @PostMapping("insertShopDTO")
    public Result insertShop(@RequestBody InsertShopDTO insertShopDTO){
        User user = getinfo();
        if(null == user){
            return Result.error("请先登录");
        }
        /**
         * 添加购物车表
         */
        insertShopDTO.setUid(user.getUid());
        exService.insertShop(insertShopDTO);
        return Result.success();
    }

    /**
     * 添加秒杀规则
     * @param rob
     * @return
     */
    @PostMapping("insertRobDTO")
    public Result insertRobDTO(@RequestBody Rob rob){
        exService.insertRobDTO(rob);
        return Result.success();
    }


    /**
     * 购物车结算
     * @param updateUserMoneyDTO
     * @return
     */
    @PostMapping("updateUserMoneyDTO")
    public Result updateUserMoneyDTO(@RequestBody UpdateUserMoneyDTO updateUserMoneyDTO){
        exService.updateUserMoney(updateUserMoneyDTO);
        return Result.success();
    }

    /**
     * 同步数据
     * @return
     */
    @PostMapping("list")
    public Result<List<Goods>> list(){
        return Result.success(exService.list());
    }



    public User getinfo() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String jsonStr = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(jsonStr, User.class);
        return user;
    }



}
