package com.bwie.expressage.service.impl;

import com.alibaba.fastjson.JSON;
import com.bwie.common.constants.TokenConstants;
import com.bwie.common.domain.Goods;
import com.bwie.common.domain.Rob;
import com.bwie.common.domain.Shop;
import com.bwie.common.domain.User;
import com.bwie.common.domain.request.*;
import com.bwie.common.domain.response.RobListVO;
import com.bwie.common.domain.response.ShopListVO;
import com.bwie.common.exception.CustomException;
import com.bwie.common.result.Result;
import com.bwie.common.utils.JwtUtils;
import com.bwie.expressage.mapper.ExMapper;
import com.bwie.expressage.service.ExService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.StringRedisTemplate;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.servlet.http.HttpServletRequest;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@Service
public class ExServiceImpl implements ExService {
    @Autowired
    private ExMapper exMapper;
    @Autowired
    private StringRedisTemplate redisTemplate;
    @Autowired
    private HttpServletRequest request;
    /**
     * 秒杀列表
     * @param robListDTO
     * @return
     */
    @Override
    public List<RobListVO> robList(RobListDTO robListDTO) {
        return exMapper.robList(robListDTO);
    }

    /**
     * 商品列表
     * @param goodsListDTO
     * @return
     */
    @Override
    public List<Goods> goodsList(GoodsListDTO goodsListDTO) {
        return exMapper.goodsList(goodsListDTO);
    }

    /**
     * 购物车列表
     * @param shopListDTO
     * @return
     */
    @Override
    public List<ShopListVO> shopList(ShopListDTO shopListDTO) {
        return exMapper.shopList(shopListDTO);
    }

    /**
     * 添加购物车表
     * @param insertShopDTO
     */
    @Transactional
    @Override
    public void insertShop(InsertShopDTO insertShopDTO) {
            if(insertShopDTO.getIsRob()==1){
                //参与秒杀
                if(insertShopDTO.getStaTime().getTime() > new Date().getTime() || new Date().getTime() > insertShopDTO.getEndTime().getTime()){
                    // 不在范围时间内
                    throw new CustomException("不在活动时间,不能进行秒杀");
                }
                // 正在参与秒杀 => 单价-10
                insertShopDTO.setPrice(insertShopDTO.getPrice()-10);
                // 判断秒杀限量还有没有
                if(insertShopDTO.getLimited()<insertShopDTO.getNum()){
                    throw new CustomException("秒杀限量库存不够");
                }
                // 修改秒杀限量 -1
                exMapper.UpdateRobLimited(insertShopDTO.getRid());
            }

        if(insertShopDTO.getNum()>insertShopDTO.getStash()){
            throw new CustomException("库存不够");
        }
        // 没有参与秒杀 / 已过期 => 不变
           //总金额               // 商品数量                 // 商品单价
       insertShopDTO.setSum( insertShopDTO.getNum() * insertShopDTO.getPrice() );
       //通过 商品id查询购物车里面有没有对应的商品有就直接修改购物车数量为+1
        UpdateShopNumDTO updateShopNumDTO = new UpdateShopNumDTO();
        updateShopNumDTO.setGid(insertShopDTO.getGid());
        updateShopNumDTO.setNum(insertShopDTO.getNum()+1);
        // 通过商品id查看该商品存不存在
        List<Goods> list = exMapper.selectGoodsByGid(insertShopDTO.getGid());
        if(list.size()>0){
            // 修改购物车对应商品数量+1
            exMapper.UpdateShopNum(updateShopNumDTO);
        }else {
            // 没有对应的商品 => 添加
            exMapper.insertShop(insertShopDTO);
        }
    }

    /**
     * 购物车结算
     * @param updateUserMoneyDTO
     */
    @Transactional
    @Override
    public void updateUserMoney(UpdateUserMoneyDTO updateUserMoneyDTO) {
        User user = getinfo();
        if(null == user){
            throw new CustomException("请先登录");
        }
        double sum1 = 0;
        List<Integer> ids = new ArrayList<>();
        sum1 = updateUserMoneyDTO.getSum();
        ids.add(updateUserMoneyDTO.getGid());
//        Shop shop1 = updateUserMoneyDTO.getShop();
//        List<Shop> shopList = updateUserMoneyDTO.getShopList();
//
//        if(null == shop1){
//            for (Shop shop : shopList) {
//                sum1 += shop.getSum();
//                ids.add(shop.getGid());
//            }
//        }else {
//            sum1 = shop1.getSum();
//            ids.add(shop1.getGid());
//        }

        if(user.getMoney()<sum1){
            throw new CustomException("余额不足");
        }
        updateUserMoneyDTO.setUid(user.getUid());
        updateUserMoneyDTO.setMoney(updateUserMoneyDTO.getMoney()-sum1);
        // 修改用户余额
        exMapper.updateUserMoney(updateUserMoneyDTO);
        // 批量修改商品库存-1
        exMapper.UpdateGoodsStashByGid(ids);
        // 删除 购物车表
        exMapper.updateShopIsdel(updateUserMoneyDTO.getGid());
    }

    /**
     * 商品列表
     * @return
     */
    @Override
    public List<Goods> list() {
        return exMapper.list();
    }

    /**
     * 添加秒杀规则
     * Required request body is missing: public com.bwie.common.result.Result com.bwie.expressage.controller.ExController.insertRobDTO(com.bwie.common.domain.Rob)
     * @param rob
     */
    @Override
    public void insertRobDTO(Rob rob) {
        rob.setStaTime(new Date());
        rob.setEndTime(new Date());
        exMapper.insertRobDTO(rob);
    }

    public User getinfo() {
        String token = request.getHeader(TokenConstants.TOKEN);
        String userKey = JwtUtils.getUserKey(token);
        String jsonStr = redisTemplate.opsForValue().get(TokenConstants.LOGIN_TOKEN_KEY + userKey);
        User user = JSON.parseObject(jsonStr, User.class);
        return user;
    }
}
